@@ -4,6 +4,8 @@ Classes for querying which lines have changed based on a diff.
 from __future__ import unicode_literals
 from abc import ABCMeta, abstractmethod
 from diff_cover.git_diff import GitDiffError
+import fnmatch
+import os
 import re
 
 
@@ -13,13 +15,15 @@ class BaseDiffReporter(object):
     """
 
     __metaclass__ = ABCMeta
+    _exclude = None
 
-    def __init__(self, name):
+    def __init__(self, name, exclude=None):
         """
         Provide a `name` for the diff report, which will
         be included in the diff coverage report.
         """
         self._name = name
+        self._exclude = exclude
 
     @abstractmethod
     def src_paths_changed(self):
@@ -48,23 +52,81 @@ class BaseDiffReporter(object):
         """
         return self._name
 
+    def _fnmatch(self, filename, patterns, default=True):
+        """Wrap :func:`fnmatch.fnmatch` to add some functionality.
+
+        :param str filename:
+            Name of the file we're trying to match.
+        :param list patterns:
+            Patterns we're using to try to match the filename.
+        :param bool default:
+            The default value if patterns is empty
+        :returns:
+            True if a pattern matches the filename, False if it doesn't.
+            ``default`` if patterns is empty.
+        """
+        if not patterns:
+            return default
+        return any(fnmatch.fnmatch(filename, pattern) for pattern in patterns)
+
+    def _is_path_excluded(self, path):
+        """
+        Check if a path is excluded.
+
+        :param str path:
+            Path to check against the exclude patterns.
+        :returns:
+            True if there are exclude patterns and the path matches,
+            otherwise False.
+        """
+        exclude = self._exclude
+        if not exclude:
+            return False
+        basename = os.path.basename(path)
+        if self._fnmatch(basename, exclude):
+            return True
+
+        absolute_path = os.path.abspath(path)
+        match = self._fnmatch(absolute_path, exclude)
+        return match
+
 
 class GitDiffReporter(BaseDiffReporter):
     """
     Query information from a Git diff between branches.
     """
 
-    def __init__(self, compare_branch='origin/master', git_diff=None):
+    def __init__(self, compare_branch='origin/master', git_diff=None,
+                 ignore_staged=None, ignore_unstaged=None,
+                 supported_extensions=None,
+                 exclude=None):
         """
         Configure the reporter to use `git_diff` as the wrapper
         for the `git diff` tool.  (Should have same interface
         as `git_diff.GitDiffTool`)
         """
-        name = "{branch}...HEAD, staged, and unstaged changes".format(branch=compare_branch)
-        super(GitDiffReporter, self).__init__(name)
+        options = list()
+        if not ignore_staged:
+            options.append("staged")
+        if not ignore_unstaged:
+            options.append("unstaged")
+
+        # Branch is always present, so use as basis for name
+        name = "{0}...HEAD".format(compare_branch)
+        if len(options) > 0:
+            # If more options are present separate them by comma's, except the last one
+            for item in options[:-1]:
+                name += ", " + item
+            # Apply and + changes to the last option
+            name += " and " + options[-1] + " changes"
+
+        super(GitDiffReporter, self).__init__(name, exclude)
 
         self._compare_branch = compare_branch
         self._git_diff_tool = git_diff
+        self._ignore_staged = ignore_staged
+        self._ignore_unstaged = ignore_unstaged
+        self._supported_extensions = supported_extensions
 
         # Cache diff information as a dictionary
         # with file path keys and line number list values
@@ -100,6 +162,18 @@ class GitDiffReporter(BaseDiffReporter):
         # If no lines modified, return an empty list
         return diff_dict.get(src_path, [])
 
+    def _get_included_diff_results(self):
+        """
+        Return a list of stages to be included in the diff results.
+        """
+        included = [self._git_diff_tool.diff_committed(self._compare_branch)]
+        if not self._ignore_staged:
+            included.append(self._git_diff_tool.diff_staged())
+        if not self._ignore_unstaged:
+            included.append(self._git_diff_tool.diff_unstaged())
+
+        return included
+
     def _git_diff(self):
         """
         Run `git diff` and returns a dict in which the keys
@@ -119,25 +193,26 @@ class GitDiffReporter(BaseDiffReporter):
 
             result_dict = dict()
 
-            for diff_str in [
-                self._git_diff_tool.diff_committed(self._compare_branch),
-                self._git_diff_tool.diff_staged(),
-                self._git_diff_tool.diff_unstaged()
-            ]:
-
+            for diff_str in self._get_included_diff_results():
                 # Parse the output of the diff string
                 diff_dict = self._parse_diff_str(diff_str)
 
                 for src_path in diff_dict.keys():
+                    if self._is_path_excluded(src_path):
+                        continue
+                    # If no _supported_extensions provided, or extension present: process
+                    root, extension = os.path.splitext(src_path)
+                    extension = extension[1:].lower()
+                    # 'not self._supported_extensions' tests for both None and empty list []
+                    if not self._supported_extensions or extension in self._supported_extensions:
+                        added_lines, deleted_lines = diff_dict[src_path]
 
-                    added_lines, deleted_lines = diff_dict[src_path]
-
-                    # Remove any lines from the dict that have been deleted
-                    # Include any lines that have been added
-                    result_dict[src_path] = [
-                        line for line in result_dict.get(src_path, [])
-                        if not line in deleted_lines
-                    ] + added_lines
+                        # Remove any lines from the dict that have been deleted
+                        # Include any lines that have been added
+                        result_dict[src_path] = [
+                            line for line in result_dict.get(src_path, [])
+                            if not line in deleted_lines
+                        ] + added_lines
 
             # Eliminate repeats and order line numbers
             for (src_path, lines) in result_dict.items():
