@@ -7,6 +7,8 @@ import subprocess
 import six
 import sys
 
+from diff_cover.command_runner import execute
+
 
 class GitPathTool(object):
     """
@@ -44,7 +46,7 @@ class GitPathTool(object):
     @classmethod
     def absolute_path(cls, src_path):
         """
-        Returns absoloute git_diff_path
+        Returns absolute git_diff_path
         """
         # If cwd is `/home/user/work/diff-cover/diff_cover`
         # and src_path is `other_package/some_file.py`
@@ -59,14 +61,5 @@ class GitPathTool(object):
         is the absolute path for the git project root.
         """
         command = ['git', 'rev-parse', '--show-toplevel', '--encoding=utf-8']
-        process = subprocess.Popen(command,
-                                   stdout=subprocess.PIPE,
-                                   stderr=subprocess.PIPE)
-        stdout, stderr = process.communicate()
-        # Git likes to add the encoding to the end of the command for some reason
-        stdout = stdout.strip()
-        git_root = stdout.split()[0] if stdout else u''
-        try:
-            return git_root.decode('utf-8')
-        except AttributeError:
-            return git_root
+        git_root = execute(command)[0]
+        return git_root.split('\n')[0] if git_root else u''
